%% 
% Run_demo_fmd--Demo code to evaluate the proposed Local Log-Euclidean 
% Multivariate Gaussian Descriptor (L^{2}EMG) with Fisher Vector Coding using 
% the Flickr material database.
% For  theoretical and technical details, please refer to the following paper:
%
% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

clear;
root = pwd;
addpath(fullfile(root,'codes\vlfeat-0.9.18\toolbox'));
addpath(fullfile(root, 'codes'));
addpath(fullfile(root, 'codes\l2emgtool'));
addpath(fullfile(root, 'codes\setupdatasets'));
addpath(fullfile(root, 'codes\svm'));
vl_setup;

S = RandStream('mt19937ar','seed',1024);
RandStream.setGlobalStream(S);
total_start_time = round(clock);

image_dataset =  'fmd';   %  scene15    caltech256   sun397  voc07
imdb = setupdataset(image_dataset,1);
idx = strfind(imdb.imageDir,'\');
imdb.imageDir = imdb.imageDir(idx+1:end);
FileList = setupFileList(image_dataset,imdb);
param = set_parameter(image_dataset);    % Set all parameters in this function

%% Diagonal L2EMG
canSkip = 1;
features = extract_SIFT_forPCA(FileList.base_image_dir,param,imdb,canSkip);
param1 = trainPCAbasis_sift(image_dataset, param, features, canSkip);
canSkip = 0;
[~] = CalculateL2EMG_diag(param1,FileList,canSkip);
gmm = train_GMM_diag_L2EMG(image_dataset,imdb,FileList.L2EMG_diag_FileList,param1);
[~] = calculateFinalRepresentationWithFV_diag_L2EMG(FileList,param1,gmm);

%% Full L2EMG
canSkip = 1;
features = extract_full_L2EMG_forPCA(FileList.base_image_dir,param,imdb,canSkip);
param2 = trainPCAbasis_full_L2EMG(image_dataset, param, features, canSkip);
canSkip = 0;
[~] = CalculateL2EMG_full(param2,FileList,canSkip);
gmm = train_GMM_full_L2EMG(image_dataset,imdb,FileList.L2EMG_full_FileList,param2);
[~] = calculateFinalRepresentationWithFV_full_L2EMG(FileList,param2,gmm);

%% SVM 
RunSVMforClassification(image_dataset,FileList);

total_end_time = round(clock);
fprintf('Total time of the classification program is:\n');
disp(total_start_time);
disp(total_end_time);

diary off ;
diary on ;
