function [L2EMG] = CalculateL2EMG_full(param, FileList,canSkip)

% CalculateL2EMG_full calculates dense L2EMG descriptors with full covariance matrices. 
%
% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

fprintf('****************************************************************\n');
fprintf('Calculate Full L2EMG \n');
fprintf('****************************************************************\n');
start_time = round(clock);
L2EMG = [];
Patch_sizes = param.Full_L2EMG.patchsize;
sample_step = param.Full_L2EMG.step;

if canSkip == 0
    parfor f =1:length(FileList.L2EMG_full_FileList)
        if ~exist(FileList.L2EMG_full_FileList{f})
            I = imread(FileList.imageFileList{f});
            [height,width,~] = size(I);
            if max(height,width) > param.maxImageSize    
                I = imresize(I, param.maxImageSize/max(height,width), 'bicubic'); 
            end

           SaveComFullL2EMG(I,Patch_sizes,sample_step,FileList.L2EMG_full_FileList{f},param.pca,param.Full_L2EMG.embeddingType);
    
        end
    end
else
    fprintf('Skipped\n');
end

fprintf('Completed.\n');
end_time = round(clock);
elapsed_time = end_time - start_time;
fprintf('Time taken: %5d days %5d hours %5d minutes %5d seconds\n\n', elapsed_time(3), elapsed_time(4), elapsed_time(5), elapsed_time(6));
end

function [L2EMG] = SaveComFullL2EMG(I, patch_size, step, currentFilePath, pca, embeddingType)

 [height,width,~] = size(I);
 [temp_fea, frames,temp_mask] = computelocalfeat(I,patch_size,step,embeddingType);   
 frames = frames';            
 L2EMG.x = frames(:,1);
 L2EMG.y = frames(:,2);
 L2EMG.mask = logical(temp_mask);
 des = bsxfun(@minus, temp_fea', pca.PCA_mean) * pca.PCA_U;
 L2EMG.des = des;
 L2EMG.imageHeight = height;
 L2EMG.imageWidth = width;
 sp_make_dir(currentFilePath);
 save(currentFilePath,'L2EMG');
 
end   






