function [ pyramid_all ] = calculateFinalRepresentationWithFV_full_L2EMG(FileList, param, gmm)

% calculateFinalRepresentationWithFV_full_L2EMG computes Fisher Vector for L2EMG descriptors 
% with full covariance matrices. 
%
% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

pyramid_all = [];
L2EMG_full_FileList = FileList.L2EMG_full_FileList;
model_full_FileList = FileList.model_full_FileList;
parfor f =1:length(FileList.L2EMG_full_FileList)
    if ~exist(model_full_FileList{f})
        [~] = calculateFV_perImage(L2EMG_full_FileList{f}, model_full_FileList{f}, param, gmm);
    end
end
end

function [ pyramid_cell ] = calculateFV_perImage(curr_L2EMGpath, curr_pyrimidpath, param, gmm)

    pyramid_cell=[];
    load(curr_L2EMGpath, 'L2EMG'); 
    hgt = L2EMG.imageHeight;
    wid = L2EMG.imageWidth;
    pyramidLevels = length(param.block_xy);
    %% Compile pyramid
    pyramid_cell = cell(pyramidLevels,1);
    for l=1:pyramidLevels
        for i=1:param.block_y(l)
            for j =1:param.block_x(l)

                 % find the coordinates of the current bin
                y_lo = floor(hgt/param.block_y(l) * (i-1));
                y_hi = floor(hgt/param.block_y(l) * i);
                x_lo = floor(wid/param.block_x(l) * (j-1));
                x_hi = floor(wid/param.block_x(l) * j);
                des = L2EMG.des( (L2EMG.x > x_lo) & (L2EMG.x <= x_hi) & ...
                                                              (L2EMG.y > y_lo) & (L2EMG.y <= y_hi) & ...
                                                              (L2EMG.mask > 0), :);
                if size(des,1) > 0
                    z = vl_fisher(des', gmm.means, gmm.covariances, gmm.priors, 'Improved') ;
                else
                    z = zeros(2*size(gmm.means,1)*size(gmm.means,2),1,'single');
                end
                pyramid_cell{l} = [pyramid_cell{l}; z];
            end
        end
    end
    pyramid_cell = cat(1,pyramid_cell{:});   
    
    sp_make_dir(curr_pyrimidpath);
    save(curr_pyrimidpath, 'pyramid_cell');

end