function features = extract_SIFT_forPCA(base_image_dir,param,imdb,canSkip)

% extract_SIFT computes the SIFT features on dense grid for all training images
%
% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

fprintf('****************************************************************\n');
fprintf('Randomly extract features for PCA basis\n');
fprintf('****************************************************************\n');
start_time = round(clock);

if canSkip == 0
    train_idx = find(imdb.images.set<=2);
    train_list = fullfile(base_image_dir,imdb.images.name(train_idx)); 
    train_image_num = length(train_idx);
    feature_num_per_image = ceil(param.pca.total_feature_num / train_image_num);
    maxImageSize = param.maxImageSize;
    parfor f =1:size(train_list,2)
        currentFdir = train_list{f};
        I = imread(currentFdir);
        [height,width,~] = size(I);
        if max(height,width) > maxImageSize    
            I = imresize(I, maxImageSize/max(height,width), 'bicubic'); 
        end
        if strcmp(param.sift.mutliScale_type,'ImageScale') == 1
            sift = getDenseSIFT(I,'step', param.sift.step, 'scales', param.sift.scales);
            temp_fea = sift.descr';
            temp_mask = (sift.contrast > param.sift.contrast)';
        else                                                          
            [RawFeas RawPos] = GenerateRawFea(I,param.sift);
            temp_fea = RawFeas';
            temp_mask = (RawPos(3,:) > param.sift.contrast)';
        end
        sel = vl_colsubset(1:size(temp_fea,1), single(feature_num_per_image)) ;
        descrs{f} = temp_fea(sel,:);
        mask{f} = temp_mask(sel);
    end
    features.fea = cat(1,descrs{:});
    features.mask = cat(1,mask{:});
else
    features = 0;
    fprintf('skipped\n');
end
end_time = round(clock);
elapsed_time = end_time - start_time;
fprintf('Time taken: %5d days %5d hours %5d minutes %5d seconds\n\n', elapsed_time(3), elapsed_time(4), elapsed_time(5), elapsed_time(6));



