function [G_filter] = GetGradientfromLib(s,ori)

% GetGradientfromLib generates different gradients for input image.

% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
%
% Peihua Li and Qilong Wang, Local Log-Euclidean covariance matrix
% (L2ECM) for image representation and its applications, in Proc.
% Eur. Conf. Comput. Vis. (ECCV), 2012, pp. 469C482.

% Please cite the papers above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.


Param.filter = s;
Param.orientation = ori;
if strcmp(Param.filter,'Frei_chen')==1
    s2 = sqrt(2);
    if strcmp(Param.orientation,'east')==1
        G_filter = [1 0 -1; s2 0 -s2; 1 0 -1]/(2+s2);
    elseif strcmp(Param.orientation,'north')==1
        G_filter=[-1 -s2 -1; 0 0 0; 1 s2 1]/(2+s2);    
    end
elseif strcmp(Param.filter,'Prewitt')==1   
    if strcmp(Param.orientation,'eastnorth')==1
        G_filter = [1 -1 -1; 1 -2 -1; 1 1 1] / 5; 
    elseif strcmp(Param.orientation,'westnorth')==1
        G_filter = [-1 -1 1; -1 -2 1 ; 1 1 1] / 5;    
    elseif strcmp(Param.orientation,'eastsouth')==1
        G_filter = [1 1 1; 1 -2 -1; 1 -1 -1] / 5; 
    elseif strcmp(Param.orientation,'westsouth')==1
        G_filter = [1 1 1; -1 -2 1; -1 -1 1] / 5;    
    end
elseif strcmp(Param.filter,'Kirsch')==1  
    if strcmp(Param.orientation,'eastnorth')==1
        G_filter = [-3 -3 -3; 5 0 -3; 5 5 -3] / 15; 
    elseif strcmp(Param.orientation,'westnorth')==1
        G_filter = [-3 -3 -3; -3 0 5 ; -3 5 5] / 15;    
    elseif strcmp(Param.orientation,'eastsouth')==1
        G_filter = [5 5 -3; 5 0 -3 ; -3 -3 -3] / 15;  
    elseif strcmp(Param.orientation,'westsouth')==1
        G_filter = [-3 5 5; -3 0 5; -3 -3 -3] / 15;    
    end 
elseif strcmp(Param.filter,'Robinson_3')==1 
    if strcmp(Param.orientation,'vertical')==1
        G_filter = [-1 0 1; -1 0 1; -1 0 1] / 3; 
    elseif strcmp(Param.orientation,'horizontal')==1
        G_filter = [1 1 1; 0 0 0 ; -1 -1 -1] / 3;    
    end
elseif strcmp(Param.filter,'Robinson_5')==1  
    if strcmp(Param.orientation,'vertical')==1
        G_filter = [-1 0 1; -2 0 2; -1 0 1] / 4; 
    elseif strcmp(Param.orientation,'horizontal')==1
        G_filter = [1 2 1; 0 0 0; -1 -2 1] / 4;    
    end
elseif strcmp(Param.filter,'Ne_babu')==1  %% Nevatia-Babu template gradient 5x5
    if strcmp(Param.orientation,'30')==1
        G_filter = [100 -32 -100 -100 -100; 100 78 -92 -100 -100; 100 100 0 -100 -100;100 100 92 -78 -100;100 100 100 32 -100] / 1102; 
    elseif strcmp(Param.orientation,'60')==1
        G_filter = [-100 -100 -100 -100 -100; 32 -78 -100 -100 -100; 100 92 0 -92 -100;100 100 100 78 -32;100 100 100 100 100] / 1102;     
    elseif strcmp(Param.orientation,'120')==1
        G_filter = [-100 -100 -100 -100 -100; -100 -100 -100 -78 32; -100 -92 0 92 100;-32  78 100 100 100;100 100 100 100 100] / 1102;
    elseif strcmp(Param.orientation,'150')==1
        G_filter = [-100 -100 -100 -32 100; -100 -100 -92 78 100; -100 -100 0 100 100;-100  -78 92 100 100;-100 32 100 100 100] / 1102;   
    elseif strcmp(Param.orientation,'210')==1
        G_filter = -[100 -32 -100 -100 -100; 100 78 -92 -100 -100; 100 100 0 -100 -100;100 100 92 -78 -100;100 100 100 32 -100] / 1102;     
    elseif strcmp(Param.orientation,'240')==1
        G_filter = -[-100 -100 -100 -100 -100; 32 -78 -100 -100 -100; 100 92 0 -92 -100;100 100 100 78 -32;100 100 100 100 100] / 1102;      
    elseif strcmp(Param.orientation,'300')==1
        G_filter = -[-100 -100 -100 -100 -100; -100 -100 -100 -78 32; -100 -92 0 92 100;-32  78 100 100 100;100 100 100 100 100] / 1102;   
      elseif strcmp(Param.orientation,'330')==1
        G_filter = -[-100 -100 -100 -32 100; -100 -100 -92 78 100; -100 -100 0 100 100;-100  -78 92 100 100;-100 32 100 100 100] / 1102;   
    end  
elseif strcmp(Param.filter,'FDOG')==1   %% First order difference of Gaussian 5x5
    G_filter =[1 1 0 -1 -1;2 2 0 -2 -2; 3 3 0 -3 -3; 2 2 0 -2 -2; 1 1 0 -1 -1]/18;
    if strcmp(Param.orientation,'horizontal')==1
        G_filter = G_filter';
    end
elseif strcmp(Param.filter,'TPO')==1   %% Truncated pyramid operator 7x7
    G_filter =[1 1 1 0 -1 -1 -1;1 2 2 0 -2 -2 -1;1 2 3 0 -3 -2 -1; 1 2 3 0 -3 -2 -1;1 2 3 0 -3 -2 -1;1 2 2 0 -2 -2 -1;1 1 1 0 -1 -1 -1]/34;
    if strcmp(Param.orientation,'horizontal')==1
        G_filter = G_filter';
    end
elseif strcmp(Param.filter,'BCO')==1   %% Box car operator 7x7
    G_filter =[1 1 1 0 -1 -1 -1;1 1 1 0 -1 -1 -1; 1 1 1 0 -1 -1 -1;1 1 1 0 -1 -1 -1; 1 1 1 0 -1 -1 -1; 1 1 1 0 -1 -1 -1; 1 1 1 0 -1 -1 -1]/21;
     if strcmp(Param.orientation,'horizontal')==1
        G_filter = G_filter';
    end
elseif strcmp(Param.filter,'Laplacian')==1   %% Eight-neighbor Laplacian (second-order)
    if strcmp(Param.orientation,'8')==1
        G_filter = [-1 -1 -1; -1 8 -1; -1 -1 -1]/8;
    elseif strcmp(Param.orientation,'4')==1
        G_filter=[1 -2 1; -2 4 -2; 1 -2 1]/8;    
    end
elseif strcmp(Param.filter,'Chebyshev')==1   %% Chebyshev polynomial (second-order)
    if strcmp(Param.orientation,'vertical')==1
        G_filter = [1  1 1; -2 -2 -2; 1 1 1] / 6; 
    elseif strcmp(Param.orientation,'horizontal')==1
        G_filter = [1 -2 1; 1 -2 1; 1 -2 1] / 6;    
    end
end

