function [fimage] = buildfeaturesFSOGrad(inimage)
% buildfeaturesFSOGrad generates raw feature for L2EMG\L2ECM image feature.
% Other cases, such as
% im_features = buildfeaturessift(detwindow);  
% im_features = buildfeaturesGabor(detwindow);  
% im_features = buildfeaturesGPos(detwindow); 
%
%
% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
%
% Peihua Li and Qilong Wang, Local Log-Euclidean covariance matrix
% (L2ECM) for image representation and its applications, in Proc.
% Eur. Conf. Comput. Vis. (ECCV), 2012, pp. 469C482.

% Please cite the papers above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

[height ,width ,depth] = size(inimage);

fdim = 17;
fimage = zeros(height, width, fdim);

if depth == 3
    imageT = double(rgb2gray(inimage));
else
    imageT = double(inimage);
end

filter=fspecial('gaussian', 3, 0.5); % 3 0.5
gimage = conv2(imageT, filter, 'same'); 

filter = GetGradientfromLib('Frei_chen','east');
filter1 = GetGradientfromLib('Frei_chen','north');
himage = conv2(gimage, filter, 'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,1)   = abs(himage);
fimage(:,:,2)   = abs(vimage);
shimage = conv2(himage, filter, 'same');
svimage = conv2(vimage, filter1, 'same');
fimage(:,:,3)   = abs(shimage);
fimage(:,:,4)   = abs(svimage);


filter = GetGradientfromLib('Prewitt','eastnorth');
filter1 = GetGradientfromLib('Prewitt','westnorth');
himage = conv2(gimage, filter, 'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,5)   = abs(himage);
fimage(:,:,6)   = abs(vimage);
shimage = conv2(himage, filter, 'same');
svimage = conv2(vimage, filter1, 'same');
fimage(:,:,7)   = abs(shimage);
fimage(:,:,8)   = abs(svimage);

filter = GetGradientfromLib('Kirsch','eastsouth');
filter1 = GetGradientfromLib('Kirsch','westsouth');
himage = conv2(gimage, filter, 'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,9)   = abs(himage);
fimage(:,:,10)   = abs(vimage);
shimage = conv2(himage, filter, 'same');
svimage = conv2(vimage, filter1, 'same');
fimage(:,:,11)   = abs(shimage);
fimage(:,:,12)   = abs(svimage);

filter = GetGradientfromLib('Laplacian','4');
filter1 = GetGradientfromLib('Laplacian','8');
himage = conv2(gimage, filter, 'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,13)   = abs(himage);
fimage(:,:,14)   = abs(vimage);

filter = GetGradientfromLib('Chebyshev','vertical');
filter1 = GetGradientfromLib('Chebyshev','horizontal');
himage = conv2(gimage, filter, 'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,15)   = abs(himage);
fimage(:,:,16)   = abs(vimage);

fimage(:,:,17)   = gimage;

end

