function [fimage] = buildfeaturesMixGrad(inimage)
% buildfeatures generates raw feature for L2ECM image feature.
%
% Please cite the following paper if you use the code:
%
%Peihua Li,  Qilong Wang: Local Log-Euclidean Covariance Matrix (L2ECM) for Image 
%Representation and Its Applications, (ECCV) 2012.
% For questions,  please conact:  Qilong Wang  (Email:  wangqilong.415@163.com), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

[height ,width ,depth] = size(inimage);

fdim = 23;
fimage = zeros(height, width, fdim);

if depth == 3
    imageT = double(rgb2gray(inimage));
else
    imageT = double(inimage);
end

filter=fspecial('gaussian', 3, 0.5); % 3 0.5
gimage = conv2(imageT, filter, 'same'); 

filter = GetGradientfromLib('Frei_chen','east');
filter1 = GetGradientfromLib('Frei_chen','north');
himage = conv2(gimage, filter, 'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,1)   = abs(himage);
fimage(:,:,2)   = abs(vimage);
shimage = conv2(himage, filter, 'same');
svimage = conv2(vimage, filter1, 'same');
fimage(:,:,3)   = abs(shimage);
fimage(:,:,4)   = abs(svimage);


filter = GetGradientfromLib('Prewitt','eastnorth');
filter1 = GetGradientfromLib('Prewitt','westnorth');
himage = conv2(gimage, filter, 'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,5)   = abs(himage);
fimage(:,:,6)   = abs(vimage);
shimage = conv2(himage, filter, 'same');
svimage = conv2(vimage, filter1, 'same');
fimage(:,:,7)   = abs(shimage);
fimage(:,:,8)   = abs(svimage);

filter = GetGradientfromLib('Kirsch','eastsouth');
filter1 = GetGradientfromLib('Kirsch','westsouth');
himage = conv2(gimage, filter, 'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,9)   = abs(himage);
fimage(:,:,10)   = abs(vimage);
shimage = conv2(himage, filter, 'same');
svimage = conv2(vimage, filter1, 'same');
fimage(:,:,11)   = abs(shimage);
fimage(:,:,12)   = abs(svimage);

filter = GetGradientfromLib('Laplacian','4');
filter1 = GetGradientfromLib('Laplacian','8');
himage = conv2(gimage, filter, 'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,13)   = abs(himage);
fimage(:,:,14)   = abs(vimage);

filter=fspecial('gaussian', 5, 0.5); 
gimage = conv2(imageT, filter, 'same');

filter = GetGradientfromLib('FDOG','vertical');
filter1 = GetGradientfromLib('FDOG','horizontal');
himage = conv2(gimage, filter,  'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,15)   = abs(himage);
fimage(:,:,16)   = abs(vimage);
shimage = conv2(himage, filter, 'same');
svimage = conv2(vimage, filter1, 'same');
fimage(:,:,17)   = abs(shimage);
fimage(:,:,18)   = abs(svimage);

filter = GetGradientfromLib('Ne_babu','60');
filter1 = GetGradientfromLib('Ne_babu','120');
himage = conv2(gimage, filter,  'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,19)   = abs(himage);
fimage(:,:,20)   = abs(vimage);
shimage = conv2(himage, filter, 'same');
svimage = conv2(vimage, filter1, 'same');
fimage(:,:,21)   = abs(shimage);
fimage(:,:,22)   = abs(svimage);

fimage(:,:,23)   = gimage;

end

