function [fimage] = buildfeaturesPos(inimage)
% buildfeatures generates raw feature for L2ECM image feature.
%
% Please cite the following paper if you use the code:
%
%Peihua Li,  Qilong Wang: Local Log-Euclidean Covariance Matrix (L2ECM) for Image 
%Representation and Its Applications, (ECCV) 2012.
% For questions,  please conact:  Qilong Wang  (Email:  wangqilong.415@163.com), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

[height width depth] = size(inimage);

fdim = 21;
fimage = zeros(height, width, fdim);

if depth == 3
    gimage = double(rgb2gray(inimage));
else
    gimage = double(inimage);
end

filter=fspecial('gaussian', 3, 0.5); % 3 0.5
gimage = conv2(gimage, filter, 'same'); 

filter = [-3 5 5;-3 0 5;-3 -3 -3] / 15; 
filter1 = [5 5 -3;5 0 -3;-3 -3 -3] / 15; 
himage = conv2(gimage, filter, 'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,1)   = abs(himage);
fimage(:,:,2)   = abs(vimage);
shimage = conv2(himage, filter, 'same');
svimage = conv2(vimage, filter1, 'same');
fimage(:,:,3)   = abs(shimage);
fimage(:,:,4)   = abs(svimage);


s2 = sqrt(2);
FreiChen_filter1= [1 0 -1; s2 0 -s2; 1 0 -1]/(2+s2);
FreiChen_filter2=[-1 -s2 -1; 0 0 0; 1 s2 1]/(2+s2);
himage = conv2(gimage, FreiChen_filter1, 'same');
vimage = conv2(gimage, FreiChen_filter2, 'same');
fimage(:,:,5)   = abs(himage);
fimage(:,:,6)   = abs(vimage);
shimage = conv2(himage, FreiChen_filter1, 'same');
svimage = conv2(vimage, FreiChen_filter2, 'same');
fimage(:,:,7)   = abs(shimage);
fimage(:,:,8)   = abs(svimage);


Second_order1 = [-1 -1 -1; -1 8 -1; -1 -1 -1]/8;
Second_order2 = [1 -2 1; -2 4 -2; 1 -2 1]/8;
himage = conv2(gimage, Second_order1, 'same');
vimage = conv2(gimage, Second_order2, 'same');
fimage(:,:,9)   = abs(himage);
fimage(:,:,10)   = abs(vimage);
shimage = conv2(himage, Second_order1, 'same');
svimage = conv2(vimage, Second_order2, 'same');
fimage(:,:,11)   = abs(shimage);
fimage(:,:,12)   = abs(svimage);


filter   = [1 -1 -1; 1 -2 -1; 1 1 1] / 5; 
filter1 = [-1 -1 1; -1 -2 1; 1 1 1] / 5; 
%filter   = [-3 -3 -3; -3 0  5; -3 5 5] / 15; 
%filter1 = [-3 -3 -3; 5 0 -3;  5 5 -3] / 15; 

himage = conv2(gimage, filter,    'same');
vimage = conv2(gimage, filter1,  'same');
fimage(:,:,13)   = abs(himage);
fimage(:,:,14)   = abs(vimage);
shimage = conv2(himage, filter,  'same');
svimage = conv2(vimage, filter1, 'same');
fimage(:,:,15)   = abs(shimage);
fimage(:,:,16)   = abs(svimage);


filter   = [1  1 1; -2 -2 -2; 1 1 1] / 6; 
filter1 = [1 -2 1; 1 -2 1; 1 -2 1] / 6; 

himage = conv2(gimage, filter,   'same');
vimage = conv2(gimage, filter1,  'same');
fimage(:,:,17)   = abs(himage);
fimage(:,:,18)   = abs(vimage);

fimage(:,:,19)   = gimage;


%fimage = bsxfun(@rdivide, fimage, sqrt(sum(sum((fimage .^2)))));


end

