function [feats, frames, im_mask_1D] = computelocalfeat(im, patchSize,gridSpacing,embeddingType)
%
%Generate the dense grid of L2EMG/L2ECM descriptors for each image

% Input:
% im: image
% gridSpacing: the spacing for the grid to be used when generating the L2EMG/L2ECM  descriptors
% patchSize: the patch size used for generating the L2EMG/L2ECM descriptors
% embeddingType: tpye of embedding to map the Gaussian descriptors to linear space

% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
%
% Peihua Li and Qilong Wang, Local Log-Euclidean covariance matrix
% (L2ECM) for image representation and its applications, in Proc.
% Eur. Conf. Comput. Vis. (ECCV), 2012, pp. 469C482.

% Please cite the papers above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.


%%  Extract local descriptors for input image
feats = [];
frames = [];
im_mask_1D = [];
valid = 0;
for i = 1:length(patchSize)

%%%%  'MG' for L2EMG descriptors and 'CM' for L2ECM descriptors
[siftArr im_mask   grid_x grid_y] = covcalculatornoncombination(im,patchSize(i), gridSpacing,  'MG', embeddingType); 
[my mx] = size(im_mask);   
[y x]= find(im_mask == 0); 
len = length(x);

 siftArr = reshape(siftArr, size(siftArr, 1) * size(siftArr, 2), size(siftArr, 3));
 siftArr = sign(siftArr) .* abs(siftArr).^0.5;
 [gridX, gridY] = meshgrid(grid_x, grid_y);
 features.x = gridX(:);
 features.y = gridY(:);
 features.mask = im_mask(:);
 features.data =siftArr;
 features.data = bsxfun(@rdivide, features.data, max(sqrt(sum(features.data .^2, 2)), eps(2)));
 
 valid = valid + len/length(im_mask(:));
 %% using_mask_one
 KK = repmat(features.mask,[1 size(features.data,2)]);
 features.data = features.data.*double(KK);
 

 feats = [feats features.data']; 
 sc = patchSize(i).*ones(size(features.x,1),1);
 framesT  = [features.x features.y sc];
 frames = [frames framesT'];
 
 im_mask_1D =  [im_mask_1D;features.mask];
  %%  using_mask_two
% feats = feats(:,features.mask==1);
% frames = frames(:,features.mask==1);
end
end