function [ feature, im_mask, grid_x, grid_y] = covcalculatornoncombination(im, patch_size, sampling_step, option, embeddingType)
%Generate the dense grid of L2EMG/L2ECM descriptors for each image

% Input:
% im: image
% sampling_step: the spacing for the grid to be used when generating the L2EMG/L2ECM  descriptors
% patchSize: the patch size used for generating the L2EMG/L2ECM descriptors
% option choice of L2ECM or L2EMG
% embeddingType: tpye of embedding to map the Gaussian descriptors to linear space


% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
%
% Peihua Li and Qilong Wang, Local Log-Euclidean covariance matrix
% (L2ECM) for image representation and its applications, in Proc.
% Eur. Conf. Comput. Vis. (ECCV), 2012, pp. 469C482.

% Please cite the papers above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

im_features = buildfeaturesFSOGrad(im); 

n=size(im_features,3);
if strcmp(option,'CM')==1
        ones_mat=ones(n,n);
        ones_mat = triu(ones_mat, 1) * sqrt(2) + eye(n);
        triu_idx=find(ones_mat>0);
        ones_vec = ones_mat(triu_idx);
        
        [im_P im_mask grid_x grid_y]  = dense_cov_CM(im_features, patch_size, sampling_step);
        
        log_im_P = im_P(:,:,triu_idx);      
        [g_y g_x bin] = size(log_im_P);
        log_im_P = reshape(log_im_P,g_y*g_x,bin);
        
        coef_mat = repmat(ones_vec', size(log_im_P, 1), 1);
        log_im_P = log_im_P .*   coef_mat;
        feature = log_im_P;
       
        feature = reshape(feature, size(im_P, 1), size(im_P, 2), size(feature, 2));
elseif strcmp(option,'MG')==1
        ones_mat=ones(n+1,n+1);
        ones_mat = triu(ones_mat, 1) * sqrt(2) + eye(n+1);
        triu_idx=find(ones_mat>0);
        ones_vec = ones_mat(triu_idx);
        
        [im_P im_mask grid_x grid_y]  = dense_cov_MG(im_features, patch_size, sampling_step, embeddingType);
        
        log_im_P = im_P(:,:,triu_idx);      
        [g_y g_x bin] = size(log_im_P);
        log_im_P = reshape(log_im_P,g_y*g_x,bin);
 
        coef_mat = repmat(ones_vec', size(log_im_P, 1), 1);
        log_im_P = log_im_P .*   coef_mat;
        feature = log_im_P;
       
        feature = reshape(feature, size(im_P, 1), size(im_P, 2), size(feature, 2));
end

end