function [sift_logP,sift_cov_mask, grid_x, grid_y] = dense_cov_MG(I_orientation, patch_size, grid_spacing, embeddingType)
%Generate the dense grid of L2EMG descriptors for each image

% Input:
% I_orientation: raw features
% gridSpacing: the spacing for the grid to be used when generating the L2EMG  descriptors
% patchSize: the patch size used for generating the L2EMG descriptors
% embeddingType: tpye of embedding to map the Gaussian descriptors to linear space

% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816

% Please cite the papers above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

[hgt wid num_angles] = size(I_orientation);
num_bins = patch_size;

grid_x = patch_size/2:grid_spacing:wid-patch_size/2;
grid_y = patch_size/2:grid_spacing:hgt-patch_size/2;

[sample_x, sample_y] = meshgrid(linspace(1,patch_size+1,num_bins+1));
sample_x = sample_x(1:num_bins,1:num_bins); sample_x = sample_x(:)-patch_size/2;
sample_y = sample_y(1:num_bins,1:num_bins); sample_y = sample_y(:)-patch_size/2;


sift_arr = zeros([length(grid_y) length(grid_x) num_angles*num_bins*num_bins]);
sift_logP = zeros([length(grid_y) length(grid_x) (num_angles+1)*(num_angles+1)]);
b = 0;
for n = 1:num_bins*num_bins
    sift_arr(:,:,b+1:b+num_angles) = I_orientation(grid_y+sample_y(n), grid_x+sample_x(n), :);
    b = b+num_angles;
end


one_pixel_data = zeros(num_angles*num_bins*num_bins,1);
one_pixel_mat  = zeros(num_angles, patch_size*patch_size);
data_cov       = zeros(num_angles, num_angles);
sift_cov_mask  = ones([length(grid_y) length(grid_x)],'uint8');

d = num_angles;
U0 = zeros(d, d);

U = zeros(d+1, d+1);
S = U;
V = U;
P = U;
logP = U;
diag_S = zeros(d+1, 1);

constrain =  1e-12 ; 

for y=1:size(sift_arr,1)
    for x=1:size(sift_arr,2)
        one_pixel_data = squeeze(sift_arr(y,x,:));
        one_pixel_mat  = reshape(one_pixel_data,num_angles, patch_size*patch_size);

        one_pixel_mat = one_pixel_mat'; 
        data_cov       = cov(one_pixel_mat);
        data_u = mean(one_pixel_mat);
        
        if strcmp(embeddingType,'IE-LogE-L')==1
            P = ones(d+1);    
            P(1:d, 1:d) = data_cov + data_u' * data_u; 
            P(1:d, d+1) = data_u';
            P(d+1, 1:d) = data_u;
%             [U S V] = svd(P);
%             diag_S = diag(S);
%             diag_S = sign(diag_S) .* abs(diag_S).^0.75;
%             P =  U*diag(diag_S)*U';
            const = max(det(P),constrain)^(-1/(d+1));
            P =  const.*P;

            [U S V] = svd(P);
            diag_S = diag(S);
            valid_flag = (length(find(diag_S>constrain)) == (d+1)); %%%%%% WQL 2014-06-04 more strictly bound   1e-12  1e-8 eps(4)
            sift_cov_mask(y, x) = valid_flag;
            if valid_flag 
                logP = U*diag(log(diag_S))*U';
            else
                logP = U*diag(log(max(diag_S, constrain)))*U';  %%%%%% WQL 2014-06-04 more strictly bound
            end
        elseif strcmp(embeddingType,'IE-LogE-R')==1
            [U0 p]= chol(data_cov);
            if p > 0
                sift_cov_mask(y, x) = 0;
                continue;
            end
            P = ones(d+1);
            P(1:d, 1:d) = U0'*U0; 
            P(1:d, d+1) = U0'*data_u';
            P(d+1, 1:d) = data_u*U0;
            P(d+1, d+1) = data_u*data_u' + 1;
%             [U S V] = svd(P);
%             diag_S = diag(S);
%             diag_S = sign(diag_S) .* abs(diag_S).^0.75;
%             P =  U*diag(diag_S)*U';
            const = max(det(P),constrain)^(-1/(d+1));
            P =  const.*P;

            [U S V] = svd(P);
            diag_S = diag(S);
            valid_flag = (length(find(diag_S>constrain)) == (d+1)); 
            sift_cov_mask(y, x) = valid_flag;
            if valid_flag 
                logP = U*diag(log(diag_S))*U';
            else
                logP = U*diag(log(max(diag_S, constrain)))*U'; 
            end

        elseif strcmp(embeddingType,'DE-LogE')==1
            [U0 p]= chol(data_cov);
            if p > 0
                sift_cov_mask(y, x) = 0;
                continue;
            end
            P(1:d, 1:d) = U0;
            P(1:d, d+1) = data_u;
            P(d+1, d+1) = 1;
            logP = logm(P);
        end
        
        sift_logP(y,x,:)  =  logP(:);

    end
end


