function RunSVMforClassification(image_dataset,FileList)

% RunSVMforClassification perfrom classification by fed Fisher vector with the 
% full L2EMG descriptors, diagonal L2EMG descriptors linear and  feature fusion 
% into a one-vs-all linear SVM.
%
% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

fprintf('****************************************************************\n');
fprintf('Peform SVM training and test\n');
fprintf('****************************************************************\n');
iter_max = 5;
meanAccuracy = cell(iter_max, 1);
mAP = cell(iter_max, 1);
mAP11 = cell(iter_max, 1);
for iter = 1: iter_max
    imdb = setupdataset(image_dataset,iter);  %randomly select training sample
    [meanAccuracy{iter},mAP{iter},mAP11{iter}] = SVM_training_test(imdb,FileList);
end

if iter_max > 1
    meanAccuracy = cat(1,meanAccuracy{:});
    mAP = cat(1,mAP{:});
    mAP11 = cat(1,mAP11{:});  % for Pascal VOC 2007
    
    Acc = mean(meanAccuracy,1);
    std_ = std(meanAccuracy,1);
    beta = 0:0.1:1;
    pl = plot(beta,Acc,'ro-');
    set(pl,'LineWidth',2);
    ylim([58,68])
    xlim([-0.1,1.1])
%     set(gca,'xtick',[0 0.25 0.5 0.75 1])
%     set(gca,'ytick',[59 60 61 62 63 64 65])
    set(gca,'FontSize',16);
    xlabel('Fusion parameter (\beta)')
    ylabel('Acc. (%)')

end
