function [meanAccuracy,mAP,mAP11] = SVM_training_test(imdb,FileList)


%% diagonal L2EMG
model_diag_FileList = FileList.model_diag_FileList;
load(model_diag_FileList{1});
dim = length(pyramid_cell);
num = numel(model_diag_FileList);
descrs = zeros(dim,num,'single');
for f =1:length(model_diag_FileList)
    load(model_diag_FileList{f});
    pyramid_cell = sign(pyramid_cell).*(abs(pyramid_cell)).^0.5;
    pyramid_cell = bsxfun(@rdivide,pyramid_cell,max(sqrt(sum(pyramid_cell.^2)),1e-12));
    descrs(:,f) = pyramid_cell;
end
train = find(imdb.images.set <= 2) ;
test = find(imdb.images.set == 3) ;
C = 10;
lambda = 1 / (C*numel(train)) ;
par = {'Solver', 'sdca', 'Verbose', ...
       'BiasMultiplier', 1, ...
       'Epsilon', 0.001, ...
       'MaxNumIterations', 30 * numel(train)} ;
if isfield(imdb.images, 'class')
  classRange = unique(imdb.images.class) ;
  test_label = imdb.images.class(test);
else
  classRange = 1:numel(imdb.classes.imageIds) ;
end
numClasses = numel(classRange) ;
scores_diag = cell(1, numel(classRange)) ;
ap = zeros(1, numel(classRange)) ;
ap11 = zeros(1, numel(classRange)) ;
w = cell(1, numel(classRange)) ;
b = cell(1, numel(classRange)) ;
for c = 1:numel(classRange)
  if isfield(imdb.images, 'class')
    y = 2 * (imdb.images.class == classRange(c)) - 1 ;
  else
    y = - ones(1, numel(imdb.images.id)) ;
    [~,loc] = ismember(imdb.classes.imageIds{classRange(c)}, imdb.images.id) ;
    y(loc) = 1 - imdb.classes.difficult{classRange(c)} ;
  end
  if all(y <= 0), continue ; end

  [w{c},b{c}] = vl_svmtrain(descrs(:,train), y(train), lambda, par{:}) ;
  scores_diag{c} = w{c}' * descrs + b{c} ;

  [~,~,info] = vl_pr(y(test), scores_diag{c}(test)) ;
  ap(c) = info.ap ;
  ap11(c) = info.ap_interp_11 ;
%   fprintf('class %s AP %.2f; AP 11 %.2f\n', imdb.meta.classes{classRange(c)}, ap(c) * 100, ap11(c)*100) ;
end


%% Full L2EMG
model_full_FileList = FileList.model_full_FileList;
load(model_full_FileList{1});
dim = length(pyramid_cell);
num = numel(model_full_FileList);
descrs = zeros(dim,num,'single');
for f =1:length(model_full_FileList)
    load(model_full_FileList{f});
    pyramid_cell = sign(pyramid_cell).*(abs(pyramid_cell)).^0.5;
    pyramid_cell = bsxfun(@rdivide,pyramid_cell,max(sqrt(sum(pyramid_cell.^2)),1e-12));
    descrs(:,f) = pyramid_cell;
end
train = find(imdb.images.set <= 2) ;
test = find(imdb.images.set == 3) ;
C = 10;
lambda = 1 / (C*numel(train)) ;
par = {'Solver', 'sdca', 'Verbose', ...
       'BiasMultiplier', 1, ...
       'Epsilon', 0.001, ...
       'MaxNumIterations', 30 * numel(train)} ;
if isfield(imdb.images, 'class')
  classRange = unique(imdb.images.class) ;
  test_label = imdb.images.class(test);
else
  classRange = 1:numel(imdb.classes.imageIds) ;
end
numClasses = numel(classRange) ;
scores_full = cell(1, numel(classRange)) ;
ap = zeros(1, numel(classRange)) ;
ap11 = zeros(1, numel(classRange)) ;
w = cell(1, numel(classRange)) ;
b = cell(1, numel(classRange)) ;
for c = 1:numel(classRange)
  if isfield(imdb.images, 'class')
    y = 2 * (imdb.images.class == classRange(c)) - 1 ;
  else
    y = - ones(1, numel(imdb.images.id)) ;
    [~,loc] = ismember(imdb.classes.imageIds{classRange(c)}, imdb.images.id) ;
    y(loc) = 1 - imdb.classes.difficult{classRange(c)} ;
  end
  if all(y <= 0), continue ; end

  [w{c},b{c}] = vl_svmtrain(descrs(:,train), y(train), lambda, par{:}) ;
  scores_full{c} = w{c}' * descrs + b{c} ;

  [~,~,info] = vl_pr(y(test), scores_full{c}(test)) ;
  ap(c) = info.ap ;
  ap11(c) = info.ap_interp_11 ;
%   fprintf('class %s AP %.2f; AP 11 %.2f\n', imdb.meta.classes{classRange(c)}, ap(c) * 100, ap11(c)*100) ;
end


%% fusion
mAP11 = [];
for beta = 0:0.1:1
    for c = 1:numel(classRange)
      if isfield(imdb.images, 'class')
        y = 2 * (imdb.images.class == classRange(c)) - 1 ;
      else
        y = - ones(1, numel(imdb.images.id)) ;
        [~,loc] = ismember(imdb.classes.imageIds{classRange(c)}, imdb.images.id) ;
        y(loc) = 1 - imdb.classes.difficult{classRange(c)} ;
      end
      if all(y <= 0), continue ; end
      
%       score_fused = scores_diag{c};
      score_fused = beta * scores_diag{c} + (1-beta) * scores_full{c};
      
      [~,~,info] = vl_pr(y(test), score_fused(test)) ;
      ap(c) = info.ap ;
      ap11(c) = info.ap_interp_11 ;
%       fprintf('class %s AP %.2f; AP 11 %.2f\n', imdb.meta.classes{classRange(c)}, ap(c) * 100, ap11(c)*100) ;
    end
    mAP11 = [mAP11,mean(ap11)];
end

scores_diag = cat(1,scores_diag{:}) ;
scores_full = cat(1,scores_full{:}) ;
mAP = [];
meanAccuracy = [];
for beta = 0:0.1:1
%     scores_fused = scores_diag;
    scores_fused = beta * scores_diag + (1-beta) * scores_full;
    if isfield(imdb.images, 'class')
      [~,preds] = max(scores_fused, [], 1) ;
      confusion = zeros(numClasses) ;
      for c = 1:numClasses
        sel = find(imdb.images.class == classRange(c) & imdb.images.set == 3) ;
        tmp = accumarray(preds(sel)', 1, [numClasses 1]) ;
        tmp = tmp / max(sum(tmp),1e-10) ;
        confusion(c,:) = tmp(:)' ;
      end
   else
      confusion = NaN ;
    end
   mAP = [mAP,mean(diag(confusion))];
   if isfield(imdb.images, 'class')
        meanAccuracy = [meanAccuracy,sum(preds(test) == test_label) / length(test_label)*100];
   else
        meanAccuracy = [meanAccuracy,0];
   end
end


