
function [param] = trainPCAbasis_sift(image_dataset, param0, features, canSkip)

% trainPCAbasis_sift learns the basis of training SIFT features via Principal 
% Component Analysis (PCA).  
%
% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

fprintf('****************************************************************\n');
fprintf('Training PCA basis \n');
fprintf('****************************************************************\n');
start_time = round(clock);

if nargin < 4
    error('Insufficient number of arguments');
end
base_data_dir_prefix = 'datas\feature';
FileName = [base_data_dir_prefix '\' image_dataset '_paramters_diag.mat'];

 if canSkip && size(dir(FileName), 1) > 0
       load(FileName, 'param');
       fprintf('skipped\n');
       return;
 end
param = param0;
features.fea = features.fea(features.mask,:);

if param.pca.PCAflag == 1
    PCA_U = [];
    PCA_means = [];
        if isfield(param.pca, 'PCA_U') &&  isfield(param.pca, 'PCA_mean') 
        else           
            PCA_means = mean(features.fea);
            features_cov = cov(features.fea);
            [PCA_U S V]=svd(features_cov);
            param.pca.PCA_U = PCA_U(:, 1:param.pca.dim_sift);
            diag_S = diag(S); 
            diag_S = diag_S(1:param.pca.dim_sift); 
            diag_S = 1.0 ./ max(sqrt(diag_S), eps(4));
            param.pca.PCA_U_whiten = PCA_U(:, 1:param.pca.dim_sift) * diag(diag_S);
            param.pca.PCA_mean  = PCA_means;
            features.fea = bsxfun(@minus, features.fea, param.pca.PCA_mean);
            features.fea = features.fea *  param.pca.PCA_U;
            param.pca.cov  = var(features.fea);
        end
end
sp_make_dir(FileName);
save(FileName, 'param');

end_time = round(clock);
elapsed_time = end_time - start_time;
fprintf('Time taken: %5d days %5d hours %5d minutes %5d seconds\n\n', elapsed_time(3), elapsed_time(4), elapsed_time(5), elapsed_time(6));

    
