function [gmm] = train_GMM_full_L2EMG(image_dataset,imdb,L2EMGFileList,param)

% train_GMM_full_L2EMG learned universal Gaussian Mixture Model (GMM) for coding 
% full L2EMG descriptors with Fisher Vector. 
%
% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

base_data_dir_prefix = 'datas\feature';
outFileName = [base_data_dir_prefix '\' image_dataset '_gmm_fullL2EMG.mat'];
if ~exist(outFileName)
    train_idx = find(imdb.images.set <= 2);
    train_list = L2EMGFileList(train_idx); 
    train_image_num = length(train_idx);
    feature_num_per_image = ceil(param.pca.total_feature_num / train_image_num);
    for k = 1:train_image_num
        load(train_list{k},'L2EMG');
        L2EMG.des = L2EMG.des(logical(L2EMG.mask),:);
        sel = vl_colsubset(1:size(L2EMG.des,1), single(feature_num_per_image)) ;
        descr{k} = L2EMG.des(sel,:);
    end
    descrs = cat(1,descr{:});

    v = var(descrs)' ;
    [gmm.means, gmm.covariances, gmm.priors] = ...
        vl_gmm(descrs', param.numClusters, 'verbose', ...
               'Initialization', 'kmeans', ...
               'CovarianceBound', double(max(v)*0.0001), ...
               'NumRepetitions', 1,...
               'MaxNumIterations', 30) ;

    sp_make_dir(outFileName);
    save(outFileName,'gmm');
else
    load(outFileName,'gmm');
end


